pub struct Branch<T: Config, S: ForkScopes> {
pub parent: Option<[u8; 32]>,
pub head: BlockNumberFor<T>,
pub scope: S,
pub genesis: [u8; 32],
pub counter: Vec<u32>,
}Expand description
Fork branch details pertaining to a block and the specialized scope state defined by the pallet/module for which the local fork graph exists.
Each branch represents the traversed path generation from its local root.
This root is not necessarily true genesis, but in the best-case scenario it is the original genesis ancestry.
If the next block is a direct child on top of the same path, the same branch structure is shared and only the branch head moves forward.
Direct child progression:
A -> B -> C -> D
same branch
only:
head = DIf the next block becomes a sibling block, it constitutes a fork and a new branch is created.
That new branch takes an Accrete over the previous
scope state so all further state becomes localized
to that fork path.
Sibling fork:
A -> B -> C
|-- D
|-- D'
D = original branch
D' = new sibling branch
scope(D') = scope(D).accrete()This structure also stores enough ancestry details to manually traverse:
- parent branches
- sibling branches
- nested forks
allowing full local fork graph inspection.
Fields
parent: Option<[u8; 32]>Structural parent branch.
None for synthetic recovery root branches
or initial local roots.
head: BlockNumberFor<T>Latest block height currently owned by this branch.
scope: SPallet-local fork scope carried by this branch lineage.
genesis: [u8; 32]Stable ancestry root for deterministic branch identity.
Usually this is the parent block hash of the block where this fork graph started.
Example:
- genesis block start: parent hash = [0; 32]
- mid-chain fork start: parent hash of that fork root
Shared across all sibling forks created from that same branch origin.
counter: Vec<u32>Fork lineage path from genesis.
Root:
A -> B -> C
counter = []
First sibling fork:
A -> B -> C
|-- C' [0]
Second sibling fork:
A -> B -> C
|-- C' [0]
|-- C'' [1]
Nested fork:
A -> B -> C
|-- C' [0]
|-- D' [0, 0]
|-- C'' [1]
|-- D' [1, 0]
|-- D'' [1, 1]Trait Implementations
Sourceimpl<T: Config, S> Decode for Branch<T, S>where
BlockNumberFor<T>: Decode,
S: Decode + ForkScopes,
impl<T: Config, S> Decode for Branch<T, S>where
BlockNumberFor<T>: Decode,
S: Decode + ForkScopes,
Sourcefn decode<__CodecInputEdqy: Input>(
__codec_input_edqy: &mut __CodecInputEdqy,
) -> Result<Self, Error>
fn decode<__CodecInputEdqy: Input>( __codec_input_edqy: &mut __CodecInputEdqy, ) -> Result<Self, Error>
fn decode_into<I>(
input: &mut I,
dst: &mut MaybeUninit<Self>,
) -> Result<DecodeFinished, Error>where
I: Input,
fn decode_into<I>(
input: &mut I,
dst: &mut MaybeUninit<Self>,
) -> Result<DecodeFinished, Error>where
I: Input,
fn skip<I>(input: &mut I) -> Result<(), Error>where
I: Input,
fn skip<I>(input: &mut I) -> Result<(), Error>where
I: Input,
fn encoded_fixed_size() -> Option<usize>
fn encoded_fixed_size() -> Option<usize>
Sourceimpl<T: Config, S> Encode for Branch<T, S>where
BlockNumberFor<T>: Encode,
S: Encode + ForkScopes,
impl<T: Config, S> Encode for Branch<T, S>where
BlockNumberFor<T>: Encode,
S: Encode + ForkScopes,
Sourcefn size_hint(&self) -> usize
fn size_hint(&self) -> usize
Sourcefn encode_to<__CodecOutputEdqy: Output + ?Sized>(
&self,
__codec_dest_edqy: &mut __CodecOutputEdqy,
)
fn encode_to<__CodecOutputEdqy: Output + ?Sized>( &self, __codec_dest_edqy: &mut __CodecOutputEdqy, )
fn using_encoded<R, F>(&self, f: F) -> R
fn using_encoded<R, F>(&self, f: F) -> R
fn encoded_size(&self) -> usize
fn encoded_size(&self) -> usize
impl<T: Config, S> EncodeLike for Branch<T, S>where
BlockNumberFor<T>: Encode,
S: Encode + ForkScopes,